import logging

from daftlistings.listing import Listing


class PropertyForSale(Listing):
    def __init__(self, data_from_search=None, url=None):
        super().__init__(data_from_search, url)

    @property
    def formalised_address(self):
        if self.data_from_search:
            t = self.data_from_search.find(
                "a", {"class": "PropertyInformationCommonStyles__addressCopy--link"}
            )
            address = t.text
        else:
            address = self._ad_page_content.find(
                "h1", {"class": "PropertyMainInformation__address"}
            ).text.strip()

        s = address.split("-")
        a = s[0].strip()
        if "SALE AGREED" in a:
            a = a.split()
            a = a[3:]
            a = " ".join([str(x) for x in a])
        return a.lower().title().strip()

    @property
    def price(self):
        try:
            if self.data_from_search:
                price = self.data_from_search.find(
                    "strong",
                    {"class": "PropertyInformationCommonStyles__costAmountCopy"},
                ).text
            else:
                price = self._ad_page_content.find(
                    "strong",
                    {"class": "PropertyInformationCommonStyles__costAmountCopy"},
                ).text
            return int("".join([str(s) for s in price if s.isdigit()]))
        except Exception as e:
            logging.error("Error getting price. Error message: " + e.args[0])

    @property
    def bedrooms(self):
        bedrooms = self.data_from_search.find_all(
            "div", {"class": "QuickPropertyDetails__iconContainer"}
        )[0]
        try:
            return int(bedrooms.find("span").text)
        except AttributeError:
            return int(bedrooms.find("img")["alt"][-1])

    @property
    def bathrooms(self):
        bathrooms = self.data_from_search.find_all(
            "div", {"class": "QuickPropertyDetails__iconContainer"}
        )[1]

        try:
            return int(bathrooms.find("span").text)
        except AttributeError:
            return int(bathrooms.find("img")["alt"][-1])

    @property
    def images(self):
        uls = self._ad_page_content.find("div", {"id": "pbxl_carousel"}).find("ul")
        images = []
        if uls is None:
            return
        for li in uls.find_all("li"):
            if li.find("img")["src"]:
                images.append(li.find("img")["src"])

        return images



import pandas as pd
import folium
import branca.colormap as cm


class MapVisualization:
    """
    This class takes a crawled daft dataset which consists of 
    price, longitute, latitude, number of bedrooms, number of bathrooms
    with price in asending order then generate a folium map object"""

    def __init__(self, dataset):
        self.df = dataset
        self.clean_up()
        self.map = self.create_map()
        self.color_names = ['darkblue','blue','lightblue','orange','lightred','red','darkred'] # 7 types of color
        self.color_codes = ['#0065a1', '#36a6d8','#86d9ff','#f1932e', '#ff8a7a','#ca3c2a','#9f3236']
        self.index = self.generate_color_bins()
        
    def clean_up(self):
        """Some rent are collected weekly. Convert them to monthly values"""
        df = self.df
        row_num = df.price.count()
        # check the first row
        if df.iloc[0].price * 4 < df.iloc[1].price:
            df.iloc[0, df.columns.get_loc('price')] *= 4.333
        # check the rest of the records
        for i in range(1, row_num):
            if df.iloc[i].price < df.iloc[i-1].price - 100:
                df.iloc[i, df.columns.get_loc('price')] *= 4.333
        self.df = df.astype({"price": int})

    def generate_color_bins(self):
        """Generate color bins based on the price at 1%, 4%, 14% ... percentiles"""
        prices = self.df.price
        percentiles = [0.01, 0.04, 0.14, 0.30, 0.56, 0.70, 0.84, 1.00]
        return [prices.quantile(p) for p in percentiles]

    def create_map(self):
        """Creat the initial folium map with the location of the first record"""
        lat_of_first_record = self.df.iloc[0]["latitude"]
        lon_of_first_record = self.df.iloc[0]["longitude"]
        return folium.Map(location=[lat_of_first_record, lon_of_first_record], 
                            zoom_start=12, control_scale=True)

    def color_dispatcher(self, price):
        for i in range(len(self.index)-1):
            if price <= self.index[i+1]:
                return self.color_names[i]

    def marker_icon(self, price):
        return folium.Icon(color=self.color_dispatcher(price))

    def add_markers(self):
        for index, row in self.df.iterrows():
            lat, lon, price = row['latitude'], row['longitude'], row['price']
            beds, baths = row['num_bedrooms'], row['num_bathrooms']
            popup_name = '<p>' + 'Bedrooms: ' + str(beds) + '</p>' + '<p>' + 'Bathrooms: ' + str(baths) + '</p>'
            popup_name += '<a href=" {0} "target="_blank"> link </a>'.format(row['daft_link'])
            icon = self.marker_icon(price)
            marker = folium.Marker([lat, lon], popup=popup_name, tooltip=price, icon=icon)
            marker.add_to(self.map)

    def add_colorbar(self):
        """add a colorbar at the top right corner of the map"""
        vmin, vmax = self.df['price'].quantile(0.005), self.df['price'].max()
        # set vmin to price.min() will screw the colorbar scale
        colormap = cm.StepColormap(self.color_codes, index=self.index, vmin=vmin, vmax=vmax)
        colormap.caption = 'Monthly rent (‎€)'
        self.map.add_child(colormap)

    def save(self, file):
        self.map.save(file)





import base64
import logging
import re

import html2text

from .request import Request


class Listing:
    def __init__(self, data_from_search=None, url=None):

        if isinstance(data_from_search, str):
            from bs4 import BeautifulSoup

            soup = BeautifulSoup(data_from_search)
            data_from_search = soup.div

        self.data_from_search = data_from_search
        self.url = url
        self.ad_page_content_data = None
        self.template_big_image = False

    @property
    def _ad_page_content(self):
        if self.ad_page_content_data is not None:
            return self.ad_page_content_data

        req = Request()

        if self.url:

            self.ad_page_content_data = req.get(self.url)
        else:
            self.ad_page_content_data = req.get(self.daft_link)

        return self.ad_page_content_data

    @property
    def id(self):
        try:
            return self._ad_page_content.find("input", {"id": "ad_id"})["value"]
        except Exception as e:
            try:
                return self._ad_page_content.find("li", {"id": "saved-ad"})["data-adid"]
            except:
                pass
            logging.error("Error getting id. Error message: " + e.args[0])
            return

    @property
    def description(self):
        try:
            description_div = str(
                self._ad_page_content.find("div", {"id": "description"})
            )

            pos_token = description_div.find("<!-- dont_cut_below_here -->")
            if pos_token == -1:
                return None
            return html2text.html2text(description_div[0:pos_token])
        except Exception as e:
            logging.error("Error getting description. Error message: " + e.args[0])
            try:
                # If the new template, currently in sales houses
                description_div = self._ad_page_content.find(
                    "p", {"class": "PropertyDescription__propertyDescription"}
                ).text
                return html2text.html2text(description_div)
            except Exception as e:
                logging.error("Error getting description. Error message: " + e.args[0])
                pass

    @property
    def agent_id(self):
        try:
            return self._ad_page_content.find("input", {"id": "agent_id"})["value"]
        except Exception as e:
            logging.error("Error getting agent_id. Error message: " + e.args[0])
            return

    @property
    def search_type(self):
        try:
            return self._ad_page_content.find("input", {"id": "ad_search_type"})[
                "value"
            ]
        except Exception as e:
            logging.error("Error getting search_type. Error message: " + e.args[0])

    @property
    def price_change(self):
        """
        This method returns any price change.
        :return:
        """
        try:
            if self.data_from_search:
                return self.data_from_search.find(
                    "div", {"class": "price-changes-sr"}
                ).text
            else:
                return self._ad_page_content.find(
                    "div", {"class": "price-changes-sr"}
                ).text
        except Exception as e:
            logging.error("Error getting price_change. Error message: " + e.args[0])
            return

    @property
    def upcoming_viewings(self):
        """
        Returns an array of upcoming viewings for a property.
        :return:
        """
        upcoming_viewings = []
        try:
            if self.data_from_search:
                viewings = self.data_from_search.find_all(
                    "div", {"class": "smi-onview-text"}
                )
            else:
                viewings = []
        except Exception as e:
            logging.error(
                "Error getting upcoming_viewings. Error message: " + e.args[0]
            )
            return
        for viewing in viewings:
            upcoming_viewings.append(viewing.text.strip())
        return upcoming_viewings

    @property
    def facilities(self):
        """
        This method returns the properties facilities.
        :return:
        """
        facilities = []

        list_items = self._ad_page_content.select("#facilities li")

        if len(list_items) == 0:
            list_items = self.ad_page_content_data.find_all(
                "span", {"class": "PropertyFacilities__iconText"}
            )

        if len(list_items) == 0:
            return []

        for li in list_items:
            facilities.append(li.text)

        return facilities

    @property
    def overviews(self):
        """
        This method returns the properties overviews.
        :return:
        """
        overviews = []
        try:
            list_items = self._ad_page_content.select("#overview li")
            # If the new template, currently in sales houses
            if len(list_items) == 0:
                list_items = self._ad_page_content.select(
                    ".PropertyOverview__overviewList  li"
                )
        except Exception as e:
            logging.error("Error getting overviews. Error message: " + e.args[0])
            return

        for li in list_items:
            overviews.append(li.text)
        return overviews

    @property
    def features(self):
        """
        This method returns the properties features.
        :return:
        """
        features = []
        try:
            list_items = self._ad_page_content.select("#features li")
            # If the new template, currently in sales houses
            if len(list_items) == 0:
                list_items = self._ad_page_content.select(
                    ".PropertyFeatures__featuresList li"
                )
        except Exception as e:
            logging.error("Error getting features. Error message: " + e.args[0])
            return

        for li in list_items:
            features.append(li.text)
        return features

    @property
    def hires_images(self):
        """
        This method returns the listing big image.
        :return:
        """
        try:
            uls = self._ad_page_content.find("div", {"id": "pbxl_carousel"})
        except Exception as e:
            logging.error("Error getting hires_image. Error message: " + e.args[0])
            return
        hires_images = []
        if uls is None:
            return
        for li in uls.find_all("li", {"class": "pbxl_carousel_item"}):
            if li.find("img")["src"]:
                hires_images.append(li.find("img")["src"])

        return hires_images

    @property
    def agent(self):
        """
        This method returns the agent name.
        :return:
        """
        try:
            if self.data_from_search:
                agent = self.data_from_search.find("ul", {"class": "links"}).text
                return agent.split(":")[1].strip()
            else:
                return self._ad_page_content.find(
                    "a", {"id": "smi-link-branded"}
                ).text.strip()
        except Exception as e:
            logging.error("Error getting agent. Error message: " + e.args[0])
            return

    @property
    def agent_url(self):
        """
        This method returns the agent's url.
        :return:
        """
        try:
            if self.data_from_search:
                agent = self.data_from_search.find("ul", {"class": "links"})
                links = agent.find_all("a")
                return links[1]["href"]
            else:
                return self._ad_page_content.find("a", {"id": "smi-link-branded"})[
                    "href"
                ]
        except Exception as e:
            logging.error("Error getting agent_url. Error message: " + e.args[0])
            return

    @property
    def contact_number(self):
        """
        This method returns the contact phone number.
        :return:
        """
        try:
            number = self._ad_page_content.find("button", {"class": "phone-number"})
            try:
                return (base64.b64decode(number.attrs["data-p"])).decode("ascii")
            except Exception as e:
                logging.error(
                    "Error getting contact_number. Error message: " + e.args[0]
                )
                return number.attrs["data-p"]
        except Exception as e:
            logging.error("Error getting contact_number. Error message: " + e.args[0])
            return "N/A"

    @property
    def daft_link(self):
        """
        This method returns the url of the listing.
        :return:
        """
        try:
            if self.data_from_search:
                link = self.data_from_search.find("a", href=True)
                return "http://www.daft.ie" + link["href"]
            else:
                return self._ad_page_content.find("link", {"rel": "canonical"})["href"]
        except Exception as e:
            logging.error("Error getting daft_link. Error message: " + e.args[0])
            return

    @property
    def shortcode(self):
        """
        This method returns the shortcode url of the listing.
        :return:
        """
        try:
            div = self._ad_page_content.find("div", {"class": "description_extras"})
            index = [i for i, s in enumerate(div.contents) if "Shortcode" in str(s)][
                0
            ] + 1
            return div.contents[index]["href"]
        except Exception as e:
            logging.error("Error getting shortcode. Error message: " + e.args[0])
            return "N/A"

    @property
    def date_insert_update(self):
        """
        This method returns the shortcode url of the listing.
        :return:
        """
        try:
            div = self._ad_page_content.find("div", {"class": "description_extras"})
            index = [
                i for i, s in enumerate(div.contents) if "Entered/Renewed" in str(s)
            ][0] + 1
            return re.search(
                "([0-9]{1,2}/[0-9]{1,2}/[0-9]{4})", str(div.contents[index])
            )[0]
        except Exception as e:
            logging.error(
                "Error getting date_insert_update. Error message: " + e.args[0]
            )
            return "N/A"

    @property
    def views(self):
        """
        This method returns the "Property Views" from listing.
        :return:
        """
        try:
            div = self._ad_page_content.find("div", {"class": "description_extras"})
            index = [
                i for i, s in enumerate(div.contents) if "Property Views" in str(s)
            ][0] + 1
            return int("".join(list(filter(str.isdigit, div.contents[index]))))
        except Exception as e:
            logging.error("Error getting views. Error message: " + e.args[0])
            return "N/A"

    @property
    def property_type(self):
        """
        This method returns the property type.
        :return:
        """
        try:
            if self.data_from_search:
                info = self.data_from_search.find(
                    "div", {"class": "QuickPropertyDetails__propertyType"}
                ).text.strip()
            else:
                return self._ad_page_content.find(
                    "div", {"class": "QuickPropertyDetails__propertyType"}
                ).text.strip()

        except Exception as e:
            logging.error("Error getting property type. Error message: " + e.args[0])
            return

    @property
    def posted_since(self):
        """
        This method returns the date the listing was entered.
        :return:
        """
        try:
            if self.data_from_search:
                info = self.data_from_search.find("div", {"class": "date_entered"}).text
                s = info.split(":")
                return s[-1].strip()
            else:
                div = self._ad_page_content.find("div", {"class": "description_extras"})
                index = [
                    i for i, s in enumerate(div.contents) if "Entered/Renewed" in str(s)
                ][0] + 1
                return re.search(
                    "([0-9]{1,2}/[0-9]{1,2}/[0-9]{4})", str(div.contents[index])
                )[0]
        except Exception as e:
            logging.error("Error getting posted_since. Error message: " + e.args[0])
            return

    @property
    def city_center_distance(self):
        """
        This method gets the distance to city center, in km.
        :return:
        """
        try:
            infos = self._ad_page_content.find_all("div", {"class": "map_info_box"})
            for info in infos:
                if "Distance to City Centre" in info.text:
                    distance_list = re.findall(
                        "Distance to City Centre: (.*) km", info.text
                    )
                    return distance_list[0]
            return None
        except Exception as e:
            logging.error(e.args[0])
            return "N/A"

    @property
    def transport_routes(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        routes = {}
        try:
            big_div = self._ad_page_content.find(
                "div", {"class": "half_area_box_right"}
            )
            uls = big_div.find("ul")
            if uls is None:
                return None
            for li in uls.find_all("li"):
                route_li = li.text.split(":")
                routes[route_li[0]] = [x.strip() for x in route_li[1].split(",")]
            return routes
        except Exception as e:
            logging.error(e.args[0])
            return "N/A"

    @property
    def latitude(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        try:
            scripts = self._ad_page_content.find_all("script")
            for script in scripts:
                if "setLngLat" in script.text:
                    find_list = re.findall(r"\.setLngLat\(\[(.*)\]", script.text)
                    if len(find_list) >= 1:
                        return find_list[0].split(", ")[1]
            return None
        except Exception as e:
            logging.error("Error getting latitude. Error message: " + e.args[0])
            return None

    @property
    def longitude(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        try:
            scripts = self._ad_page_content.find_all("script")
            for script in scripts:
                if "setLngLat" in script.text:
                    find_list = re.findall(r"\.setLngLat\(\[(.*)\]", script.text)
                    if len(find_list) >= 1:
                        return find_list[0].split(", ")[0]
            return None
        except Exception as e:
            logging.error("Error getting longitude. Error message: " + e.args[0])
            return None

    @property
    def ber_code(self):
        """
        This method gets ber code listed in Daft.
        :return:
        """
        try:
            alt_text = self._ad_page_content.find("span", {"class": "ber-hover"}).find(
                "img"
            )["alt"]

            if "exempt" in alt_text:
                return "exempt"
            else:
                alt_arr = alt_text.split()
                if "ber" in alt_arr[0].lower():
                    return alt_arr[1].lower()
                else:
                    return None
        except Exception as e:
            logging.error("Error getting the Ber Code. Error message: " + e.args[0])
            return None

    @property
    def commercial_area_size(self):
        """
        This method returns the area size. This method should only be called when retrieving commercial type listings.
        :return:
        """
        try:
            if self.data_from_search:
                info = self.data_from_search.find("ul", {"class": "info"}).text
                s = info.split("|")
                return s[1].strip()
            else:
                return
        except Exception as e:
            logging.error(
                "Error getting commercial_area_size. Error message: " + e.args[0]
            )
            return "N/A"

    @property
    def advertiser_name(self):
        """
        This method returns the area size. This method should only be called when retrieving commercial type listings.
        :return:
        """
        try:
            return (
                self._ad_page_content.find("div", {"id": "smi-negotiator-photo"})
                .find("h2")
                .text
            )
        except Exception as e:
            logging.error(
                "Error getting commercial_area_size. Error message: " + e.args[0]
            )
            return "N/A"

    @property
    def price_change_history(self):
        price_changes = []
        try:
            prices = self._ad_page_content.find_all(
                "div", {"class": "PropertyPriceHistory__propertyPriceEntryContainer"}
            )
            for price in prices:
                date = price.find(
                    "div", {"class": "PropertyPriceHistory__propertyPriceDate"}
                )
                price_at_date = price.find(
                    "div", {"class": "PropertyPriceHistory__propertyPrice"}
                )
                price_changes.append(
                    {"date": date.text.strip(), "price": price_at_date.text.strip()}
                )

        except Exception as e:
            logging.error("Error getting price_change. Error message: " + e.args[0])
            return []
        return price_changes

    @property
    def contact_info(self):
        """
        This method returns the area size. This method should only be called when retrieving commercial type listings.
        :return:
        """
        try:
            return (
                self._ad_page_content.find("div", {"class": "smi-contact-numbers"})
                .find("div", {"class": "phone-info"})
                .text.strip()
            )
        except Exception as e:
            logging.error(
                "Error getting commercial_area_size. Error message: " + e.args[0]
            )
            return "N/A"

    def contact_advertiser(self, name, email, contact_number, message):
        """
        This method allows you to contact the advertiser of a listing.
        :param name: Your name
        :param email: Your email address.
        :param contact_number: Your contact number.
        :param message: Your message.
        :return:
        """

        req = Request()

        ad_search_type = self.search_type
        agent_id = self.agent_id
        ad_id = self.id

        response = req.post(
            "https://www.daft.ie/ajax_endpoint.php?",
            params={
                "action": "daft_contact_advertiser",
                "from": name,
                "email": email,
                "message": message,
                "contact_number": contact_number,
                "type": ad_search_type,
                "agent_id": agent_id,
                "id": ad_id,
                "self_copy": 1,
            },
        )
        return response.status_code == 200

    def as_dict(self):
        """
        Return a Listing object as Dictionary
        :return: dict
        """
        return {
            "search_type": self.search_type,
            "agent_id": self.agent_id,
            "id": self.id,
            "price": self.price,
            "price_change": self.price_change,
            "viewings": self.upcoming_viewings,
            "facilities": self.facilities,
            "overviews": self.overviews,
            "formalised_address": self.formalised_address,
            "listing_image": self.images,
            "listing_hires_image": self.hires_images,
            "agent": self.agent,
            "agent_url": self.agent_url,
            "contact_number": self.contact_number,
            "contact_info": self.contact_info,
            "advertiser_name": self.advertiser_name,
            "daft_link": self.daft_link,
            "shortcode": self.shortcode,
            "date_insert_update": self.date_insert_update,
            "views": self.views,
            "description": self.description,
            "dwelling_type": self.property_type,
            "posted_since": self.posted_since,
            "num_bedrooms": self.bedrooms,
            "num_bathrooms": self.bathrooms,
            "city_center_distance": self.city_center_distance,
            "transport_routes": self.transport_routes,
            "latitude": self.latitude,
            "longitude": self.longitude,
            "ber_code": self.ber_code,
            "commercial_area_size": self.commercial_area_size,
        }

    def as_dict_for_mapping(self):
        return {
            "price": self.price,
            "formalised_address": self.formalised_address,
            "daft_link": self.daft_link,
            "num_bedrooms": self.bedrooms,
            "num_bathrooms": self.bathrooms,
            "latitude": self.latitude,
            "longitude": self.longitude,
        }

    def __repr__(self):
        return "Listing (%s)" % self.formalised_address



import requests
from bs4 import BeautifulSoup

from .exceptions import DaftException


class Request:
    def __init__(self):
        self._headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:24.0) Gecko/20100101 Firefox/24.0"
        }

    def get(self, url, params=None):
        req = requests.get(url, headers=self._headers, params=params)
        if req.status_code != 200:
            raise DaftException(reason=req.reason)

        soup = BeautifulSoup(req.content, "html.parser")
        return soup

    def post(self, url, params=None):
        req = requests.post(url, params=params, headers=self._headers)
        if req.status_code != 200:
            raise DaftException(reason=req.reason)

        return req



from enum import Enum


class PropertyType(Enum):
    APARTMENT = "&s%5Bpt_id%5D%5B0%5D=1"
    HOUSE = "&s%5Bpt_id%5D%5B1%5D=2"
    STUDIO = "&s%5Bpt_id%5D%5B2%5D=3"
    FLAT = "&s%5Bpt_id%5D%5B3%5D=4"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SaleType: %s>" % self


class HouseType(Enum):
    DETACHED = "s%5Bhouse_type%5D=detached"
    SEMI_DETACHED = "&s%5Bhouse_type%5D=semi-detached"
    TERRACED = "&s%5Bhouse_type%5D=terraced"
    END_OF_TERRACE = "&s%5Bhouse_type%5D=end-of-terrace"
    TOWNHOUSE = "&s%5Bhouse_type%5D=townhouse"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<HouseType: %s>" % self


class SaleType(Enum):
    HOUSES = "/houses-for-sale/"
    PROPERTIES = "/property-for-sale/"
    AUCTION = "/houses-for-auction/"
    APARTMENTS = "/apartments-for-sale/"
    COMMERCIAL = "/commercial-property/"
    NEW = "/new-homes-for-sale/"
    OVERSEAS = "/overseas-property-for-sale/"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SaleType: %s>" % self


class CommercialType(Enum):
    OFFICE = "/offices/"
    RETAIL = "/retail-units/"
    OFFICE_SHARE = "/office-share/"
    INDUSTRIAL_UNIT = "/industrial-unit/"
    COMMERCIAL_SITE = "/commercial-site/"
    AGRICULTURAL_LAND = "/agricultural-farm-land/"
    RESTAURANT_BAR_HOTEL = "/restaurant-hotel-bar/"
    INDUSTRIAL_SITE = "/industrial_site/"
    DEV_LAND = "/development-land/"
    INVESTMENT_PROPERTY = "/investment-property/"
    SERVICED_OFFICE = "/serviced-office/"
    OVERSEAS = "/overseas-commercial-property-for-sale/"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<CommercialType: %s>" % self


class RentType(Enum):
    HOUSES = "/houses-for-rent/"
    APARTMENTS = "/apartments-for-rent/"
    ANY = "/residential-property-for-rent/"
    STUDIO = "/studio-apartments-for-rent/"
    FLAT = "/flats-for-rent/"
    SHORT_TERM = "/short-term-rentals/"
    STUDENT_ACCOMMODATION = "/student-accommodation/"
    PARKING_SPACES = "/parking-spaces/"
    ROOMS_TO_SHARE = "/rooms-to-share/"
    FLAT_TO_SHARE = "/flat-to-share/"
    APARTMENT_TO_SHARE = "/apartment-share/"
    HOUSE_SHARE = "/house-share/"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<RentType: %s>" % self


class StudentAccommodationType(Enum):
    ROOMS_TO_SHARE = "/rooms-to-share/"
    APARTMENTS = "/apartments-for-rent/"
    ANY = "/residential-property-for-rent/"
    STUDIO = "/studio-apartments-for-rent/"
    FLAT = "/flats-for-rent/"
    SHORT_TERM = "/short-term-rentals/"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<StudentAccommodationType: %s>" % self


class RoomType(Enum):
    SINGLE = "single"
    DOUBLE = "double"
    TWIN_ROOM = "twin"
    SHARED = "shared"
    SINGLE_OR_DOUBLE = "own"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<RoomType: %s>" % self


class QueryParam(Enum):
    SALE_AGREED = "&s[area_type]=on&s[agreed]=1&s[advanced]=1"
    SALE_AGREED_WITH_PRICE = "&s%5Bagreed%5D=1&s%5Badvanced%5D=1"
    MIN_PRICE = "&s%5Bmnp%5D="
    MAX_PRICE = "&s%5Bmxp%5D="
    IGNORED_AGENTS = "&s%5Bignored_agents%5D%5B1%5D"
    AVALIABILITY = "&s%5Bavailable_for%5D="
    MIN_BEDS = "&s%5Bmnb%5D="
    MAX_BEDS = "&s%5Bmxb%5D="
    SORT_BY = "&s%5Bsort_by%5D="
    SORT_ORDER = "&s%5Bsort_type%5D="
    COMMERCIAL_MIN = "&s%5Bmin_size%5D="
    COMMERCIAL_MAX = "&s%5Bmax_size%5D="
    OPEN_VIEWING = "&s%5Bopenviewing%5D=1"
    WITH_PHOTOS = "&s%5Bphotos%5D=1"
    KEYWORDS = "&s%5Btxt%5D="
    FURNISHED = "&s%5Bfurn%5D=1"
    COUPLES_ACCEPTED = "&s%5Bcouples%5D=1"
    ENSUITE_ONLY = "&s%5Bes%5D=1"
    ROOM_TYPE = "&s%5Broom_type%5D="
    GENDER = "&s%5Bgender%5D="
    ADDRESS = "&s%5Baddress%5D="
    ADVANCED = "&s%5Badvanced%5D=1"
    MIN_LEASE = "&s%5Bmin_lease%5D="
    MAX_LEASE = "&s%5Bmax_lease%5D="
    DAYS_OLD = "&s%5Bdays_old%5D="
    NUM_OCCUPANTS = "&s%5Boccupants%5D="
    ROUTE_ID = "&s%5Broute_id%5D="
    FIND_TEAMUPS = "&submit_search=Find+Teamups+%BB"
    PETS_ALLOWED = "&s%5Bfacilities%5D%5Bf9%5D=512"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<QueryParam: %s>" % self


class Gender(Enum):
    MALE = "male"
    FEMALE = "female"
    EITHER = "on"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<Gender: %s>" % self


class SortOrder(Enum):
    ASCENDING = "a"
    DESCENDING = "d"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SortOrder: %s>" % self


class SortType(Enum):
    DATE = "date"
    DISTANCE = "distance"
    PRICE = "price"
    UPCOMING_VIEWING = "upcoming_viewing"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SortType: %s>" % self


class University(Enum):
    NCI = "national-college-of-ireland-nci"
    UCD = "university-college-dublin-nui"
    GCD = "griffith-college-dublin"
    TCD = "trinity-college-university-of-dublin"
    DCU = "dublin-city-university"
    DIT = "dit-kevin-street"
    WIT = "waterford-institute-of-technology"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<University: %s>" % self


class AreaType(Enum):
    ENROUTE = "&s%5Barea_type%5D=enroute"
    TRANSPORT_ROUTE = "&s%5Barea_type%5D=trans"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<AreaType: %s>" % self


class TransportRoute(Enum):
    DART = "1"
    LUAS_TALLAGHT_LINE = "2"
    LUAS_SANDYFORD_LINE = "3"
    BUS_LINE_1 = "4"
    BUS_LINE_2 = "5"
    BUS_LINE_3 = "224"
    BUS_LINE_4 = "227"
    BUS_LINE_5 = "6"
    BUS_LINE_7 = "7"
    BUS_LINE_7A = "8"
    BUS_LINE_7B = "9"
    BUS_LINE_7D = "10"
    BUS_LINE_7N = "11"
    BUS_LINE_8 = "225"
    BUS_LINE_9 = "228"
    BUS_LINE_10 = "12"
    BUS_LINE_10A = "13"
    BUS_LINE_11 = "14"
    BUS_LINE_11A = "15"
    BUS_LINE_11B = "16"
    BUS_LINE_13 = "17"
    BUS_LINE_13A = "18"
    BUS_LINE_13B = "19"
    BUS_LINE_14 = "20"
    BUS_LINE_15 = "22"
    BUS_LINE_15A = "23"
    BUS_LINE_15B = "24"
    BUS_LINE_15C = "25"
    BUS_LINE_15D = "26"
    BUS_LINE_15E = "27"
    BUS_LINE_15F = "28"
    BUS_LINE_15N = "29"
    BUS_LINE_15X = "30"
    BUS_LINE_16 = "31"
    BUS_LINE_17 = "33"
    BUS_LINE_17A = "34"
    BUS_LINE_18 = "35"
    BUS_LINE_19 = "36"
    BUS_LINE_19A = "37"
    BUS_LINE_20B = "38"
    BUS_LINE_25 = "39"
    BUS_LINE_25A = "40"
    BUS_LINE_25N = "41"
    BUS_LINE_25X = "42"
    BUS_LINE_26 = "43"
    BUS_LINE_27 = "44"
    BUS_LINE_27B = "45"
    BUS_LINE_27C = "46"
    BUS_LINE_27N = "47"
    BUS_LINE_27X = "48"
    BUS_LINE_29A = "49"
    BUS_LINE_29N = "50"
    BUS_LINE_31 = "51"
    BUS_LINE_31A = "52"
    BUS_LINE_31B = "53"
    BUS_LINE_31N = "54"
    BUS_LINE_32 = "55"
    BUS_LINE_32A = "56"
    BUS_LINE_32B = "57"
    BUS_LINE_32X = "58"
    BUS_LINE_33 = "59"
    BUS_LINE_33A = "60"
    BUS_LINE_33B = "61"
    BUS_LINE_33N = "62"
    BUS_LINE_37 = "63"
    BUS_LINE_37X = "64"
    BUS_LINE_38 = "65"
    BUS_LINE_38A = "66"
    BUS_LINE_38B = "67"
    BUS_LINE_38C = "68"
    BUS_LINE_39 = "69"
    BUS_LINE_39A = "70"
    BUS_LINE_39B = "71"
    BUS_LINE_39N = "72"
    BUS_LINE_39X = "73"
    BUS_LINE_40 = "74"
    BUS_LINE_40A = "75"
    BUS_LINE_40B = "76"
    BUS_LINE_40C = "77"
    BUS_LINE_40D = "78"
    BUS_LINE_40N = "79"
    BUS_LINE_41 = "80"
    BUS_LINE_41A = "81"
    BUS_LINE_41B = "82"
    BUS_LINE_41C = "83"
    BUS_LINE_41N = "84"
    BUS_LINE_41X = "85"
    BUS_LINE_42 = "86"
    BUS_LINE_42A = "87"
    BUS_LINE_42B = "88"
    BUS_LINE_42N = "89"
    BUS_LINE_43 = "90"
    BUS_LINE_44 = "91"
    BUS_LINE_44B = "92"
    BUS_LINE_44C = "93"
    BUS_LINE_44N = "94"
    BUS_LINE_45 = "95"
    BUS_LINE_45A = "96"
    BUS_LINE_46 = "97"
    BUS_LINE_46A = "98"
    BUS_LINE_46B = "99"
    BUS_LINE_46C = "100"
    BUS_LINE_46D = "101"
    BUS_LINE_46E = "102"
    BUS_LINE_46N = "103"
    BUS_LINE_46X = "104"
    BUS_LINE_48A = "105"
    BUS_LINE_48N = "106"
    BUS_LINE_49 = "107"
    BUS_LINE_49A = "108"
    BUS_LINE_49N = "109"
    BUS_LINE_49X = "110"
    BUS_LINE_50 = "111"
    BUS_LINE_50X = "112"
    BUS_LINE_51 = "113"
    BUS_LINE_51A = "114"
    BUS_LINE_51B = "115"
    BUS_LINE_51C = "116"
    BUS_LINE_51D = "117"
    BUS_LINE_51N = "118"
    BUS_LINE_51X = "119"
    BUS_LINE_53 = "120"
    BUS_LINE_53A = "121"
    BUS_LINE_54A = "122"
    BUS_LINE_54N = "123"
    BUS_LINE_56A = "124"
    BUS_LINE_58C = "125"
    BUS_LINE_58X = "126"
    BUS_LINE_59 = "127"
    BUS_LINE_63 = "128"
    BUS_LINE_65 = "129"
    BUS_LINE_65B = "130"
    BUS_LINE_65X = "131"
    BUS_LINE_66 = "132"
    BUS_LINE_66A = "133"
    BUS_LINE_66B = "134"
    BUS_LINE_66N = "135"
    BUS_LINE_66X = "136"
    BUS_LINE_67 = "137"
    BUS_LINE_67A = "138"
    BUS_LINE_67N = "139"
    BUS_LINE_67X = "140"
    BUS_LINE_68 = "141"
    BUS_LINE_69 = "142"
    BUS_LINE_69N = "143"
    BUS_LINE_69X = "144"
    BUS_LINE_70 = "145"
    BUS_LINE_70N = "146"
    BUS_LINE_70X = "147"
    BUS_LINE_75 = "148"
    BUS_LINE_76 = "149"
    BUS_LINE_76A = "150"
    BUS_LINE_76B = "151"
    BUS_LINE_77 = "152"
    BUS_LINE_77A = "153"
    BUS_LINE_77B = "154"
    BUS_LINE_77N = "155"
    BUS_LINE_77X = "156"
    BUS_LINE_78 = "157"
    BUS_LINE_78A = "158"
    BUS_LINE_79 = "159"
    BUS_LINE_83 = "160"
    BUS_LINE_84 = "161"
    BUS_LINE_84N = "162"
    BUS_LINE_84X = "163"
    BUS_LINE_86 = "164"
    BUS_LINE_88N = "165"
    BUS_LINE_90 = "166"
    BUS_LINE_90A = "167"
    BUS_LINE_102 = "168"
    BUS_LINE_103 = "169"
    BUS_LINE_104 = "170"
    BUS_LINE_105 = "171"
    BUS_LINE_111 = "172"
    BUS_LINE_114 = "173"
    BUS_LINE_115 = "174"
    BUS_LINE_116 = "175"
    BUS_LINE_117 = "176"
    BUS_LINE_118 = "177"
    BUS_LINE_120 = "178"
    BUS_LINE_121 = "179"
    BUS_LINE_122 = "180"
    BUS_LINE_123 = "181"
    BUS_LINE_127 = "182"
    BUS_LINE_129 = "183"
    BUS_LINE_130 = "184"
    BUS_LINE_140 = "226"
    BUS_LINE_145 = "185"
    BUS_LINE_146 = "186"
    BUS_LINE_150 = "187"
    BUS_LINE_161 = "188"
    BUS_LINE_172 = "189"
    BUS_LINE_184 = "190"
    BUS_LINE_185 = "191"
    BUS_LINE_201 = "192"
    BUS_LINE_202 = "193"
    BUS_LINE_206 = "194"
    BUS_LINE_210 = "195"
    BUS_LINE_220 = "196"
    BUS_LINE_230 = "197"
    BUS_LINE_236 = "198"
    BUS_LINE_237 = "199"
    BUS_LINE_238 = "200"
    BUS_LINE_239 = "201"
    BUS_LINE_270 = "202"
    BUS_LINE_746 = "203"
    BUS_LINE_747 = "204"
    BUS_LINE_748 = "205"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<TransportRoute: %s>" % self



from daftlistings.listing import Listing


class PropertyForRent(Listing):
    def __init__(self, data_from_search=None, url=None):
        super().__init__(data_from_search, url)

    @property
    def price(self):
        price = self._ad_page_content.find("div", {"id": "smi-price-string"}).text
        if not price:
            price = self._ad_page_content.find("div", {"id": "PropertyInformationCommonStyles__costAmountCopy"}).text
        price_arr = price.split()
        d = int("".join([str(s) for s in price if s.isdigit()]))
        d = d * 4.345 if "week" in price_arr else d
        return float(d)

    @property
    def formalised_address(self):
        if self.data_from_search:
            t = self.data_from_search.find("div", {"class": "search_result_title_box"})
            address = t.find("a").text
        else:
            address = (
                self._ad_page_content.find("div", {"class": "smi-object-header"})
                .find("h1")
                .text.strip()
            )

        s = address.split("-")
        a = s[0].strip()
        if "SALE AGREED" in a:
            a = a.split()
            a = a[3:]
            a = " ".join([str(x) for x in a])
        return a.lower().title().strip()

    @property
    def bedrooms(self):
        info = self.data_from_search.find("ul", {"class": "info"}).text
        s = info.split("|")
        try:
            nb = s[1].strip()
            return int(nb.split()[0])
        except IndexError:
            return "N/A"

    @property
    def bathrooms(self):
        info = self.data_from_search.find("ul", {"class": "info"}).text
        s = info.split("|")
        try:
            nb = s[2].strip()
            return int(nb.split()[0])
        except IndexError:
            return "N/A"

    @property
    def images(self):
        uls = self._ad_page_content.find("ul", {"class": "smi-gallery-list"})
        images = []
        if uls is None:
            return
        for li in uls.find_all("li"):
            if li.find("img")["src"]:
                images.append(li.find("img")["src"])

        return images
